<?php
namespace Teknisa\CTB\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\CTB\Services\ContabilizacaoRetiradaService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Util\Utilities;

const msgPadrao_ObservacaoLote = 'Contabilização das retiradas';

class ContabilizacaoRetiradaController {

    protected $ctbService;

    public function __construct(EntityManager $entityManager, ContabilizacaoRetiradaService $ctbService) {
        $this->entityManager            = $entityManager;
        $this->ctbService = $ctbService;
    }

    /*Gera a contabilização de estoque - retiradas*/
    public function geraContabilizacaoRetirada(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $Result = $this->ctbService->geraLote($request->getRow(), msgPadrao_ObservacaoLote);
            $response->addDataset(new DataSet('Result', array($Result)));
        } catch(\Exception $e) {
            $this->ctbService->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*Gera a Contabilização de Ajustes e Implantações*/
    public function geraContabilizacaoAjustImpla(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $Result = $this->ctbService->sbtContabClick($request->getRow());

            $response->addDataset(new DataSet('Result', array($Result)));
        } catch(\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*Gera o relatório de Apropriação Contábil Implantações*/
    public function geraRelatorioImplantacao(DTO\Request\Row $request, DTO\Response $response) {
        try {

            $Result = $this->ctbService->Prepara_Impressao($request->getRow(), 'N');
            $row = $request->getRow();
            switch($row["RELATORIO"]) {
                case "A": //Apropriação Contábil Analítica Implantações/Ajustes
                    $response->addDataset(new DataSet('data', $Result["cdsRelatorioAnal"]));
                    break;
                case "S": //Apropriação Contábil Sintética Implantações/Ajustes
                    $response->addDataset(new DataSet('data', $Result["cdsRelatorioSin"]));
                    break;
                case "C": //Produtos sem Conta Contábil
                    $response->addDataset(new DataSet('data', $Result["cdsProdSContCtbl"]));
                    break;
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*Gera o relatório dos Ajustes de Apuração (Crédito Presumido,
     Credito 3,9% - SP, Estorno BA, Estorno RJ
    */
    public function geraRelatorioContabImpostos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            $Result = $this->ctbService->geraRelatorioContabImpostos($row);
            
            $this->entityManager->commit();

            $response->addDataset(new DataSet('data', [$Result["response"]]));
            
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*Gera o relatório do Ajuste de Apuração Fiscal */
    public function geraRelatorioAjusteApuracaoFis(DTO\Request\DataSet  $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet()->getRows();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $Result = array();
            foreach($dataset as $ajuste) {
                array_push($Result, $this->ctbService->geraRelatorioAjusteApuracaoFis($ajuste));
            }
            
            
            //$this->entityManager->commit();
            $data = $this->ctbService->buscaAjustesMapa($dataset[0]);
            $data = new DataSet('ctb_ajuste_apurfiscal', $data);
            $response->addDataSet($data);
            $response->addDataset(new DataSet('sucesso', $Result));
            
        } catch(\Exception $e) {
            try {
                 $this->entityManager->rollback();
            } catch(\Exception $e) {}
            
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /*Gera o relatório do Ajuste de Apuração Fiscal */
    public function salvarAjustesContabFiscal(DTO\Request\DataSet  $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet()->getRows();
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $ajusteapurcontr = array_filter($dataset, function($item){ return $item['IDTABELA'] == 'AC';});
            if(!empty($ajusteapurcontr)){
                foreach($ajusteapurcontr as $row){ 
                    $this->ctbService->updateAjusteapurcontr($row);
                }
            }
                    
            $itmaparecup = array_filter($dataset,  function($item){ return $item['IDTABELA'] == 'IM';});
            if(!empty($itmaparecup)){
                foreach($itmaparecup as $row){
                    $this->ctbService->updateItmaparecup($row);
                }
            }

            $itdetapucontr = array_filter($dataset,  function($item){ return $item['IDTABELA'] == 'ID';});
            if(!empty($itdetapucontr)){
                foreach($itdetapucontr as $row){
                    $this->ctbService->updateItdetapucontr($row);
                }
            }
            $connection->commit();
            $this->entityManager->commit();

            $response->addDataset(new DataSet('data', ['success']));
            
        } catch(\Exception $e) {
            try{
                 $this->entityManager->rollback();
            }catch(\Exception $e) {}
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    // Traz os ajustes relacionados a um mapa
    
    public function buscaAjustesMapa(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $row = $this->getFilterArray($request);
            $dataSet = $this->ctbService->buscaAjustesMapa($row);
            $dataSet = new DataSet('ctb_ajuste_apurfiscal', $dataSet);
            $response->addDataSet($dataSet);
           
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
       
        
    }
    
    private function getFilterArray($request) {
        $filter = array();
        $conditions = $request->getFilterCriteria()->getConditions();
        foreach ($conditions as $condition) {
            $filter[$condition["columnName"]] = $condition["value"];
        }
        return $filter;
    }
}
